/*
 *  Monitor.h
 *
 *  Created by David Lapsley on Mon Feb 23 2004.
 *
 */

/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifndef MONITOR_H
#define MONITOR_H

#include <common.h>
#include <iostream>
#include <RRD.h>


/** Monitor class
 * @doc This class is responsible for monitoring statistics and storing them
 * in a database in memory. At the end of the run, the statistics can then be
 * dumped to file. This is to provide monitoring capability while minimizing
 * the impact on  performance.
 */

class Monitor
{
	 /** Parameter.
	  * Maximum size of any array in this database.
	  */
	 const u_int32_t _sz;
	 /** Stat array.
	  * Feedback SN received from client.
	  */
	 Uint16RRD _fb_rrd;
	 /** Stat array.
	  * SNs transmitted by server (or received by client).
	  */
	 Uint16RRD _sn_rrd;
	 /** Stat rrd.
	  * Stores sleep times in the transmit loop
	  */
	 DoubleRRD _sleep_rrd;
	 /** Stat rrd.
	  * Stores stream of packets deleted from client receive buffer.
	  */
	 Uint16RRD _del_rrd;
	 /** Stat rrd.
	  * Stores maximum sequence numbers at the client receive side.
	  * (at the start of retransmission).
	  */
	 Uint16RRD _msn_rrd;
  public:
	 /** Constructor.
	  * @param sz total number of samples per rrd
	  */
	 Monitor(const unsigned int& sz=262140);
	 /** Destructor.
	  */
	 ~Monitor();
	 /** Update.
	  * Updates fb rrd.
	  * @param val New value to add to rrd.
	  */
	 inline void update_fb(const u_int16_t& val) {
		  _fb_rrd.update(val);
	 }
	 /** Update.
	  * Updates sn rrd.
	  * @param val New value to add to rrd.
	  */
	 inline void update_sn(const u_int16_t& val) {
		  _sn_rrd.update(val);
	 }
	 /** Update.
	  * Updates sleep rrd.
	  * @param val New value to add to rrd.
	  */
	 inline void update_sleep(const double& val) {
		  _sleep_rrd.update(val);
	 }
	 /** Update.
	  * Updates del rrd.
	  * @param val New value to add to rrd.
	  */
	 inline void update_del(const u_int16_t& val) {
		  _del_rrd.update(val);
	 }
	 /** Update.
	  * Updates msn rrd.
	  * @param val New value to add to rrd.
	  */
	 inline void update_msn(const u_int16_t& val) {
		  _msn_rrd.update(val);
	 }
};


#endif

