/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifndef RTPSOCKET_H
#define RTPSOCKET_H

#include <common.h>
#include <Socket.h>
#include <RTPSession.h>
#include <string>
#include <iostream>
#include <syslog.h>
#include <Mutex.h>

/** RTPSocket class
 * This class implements an RTP server socket. It inherits from VRSocketA.
 */
class RTPSocket: public Socket
{
  protected:
	 /** Communication port.
	  * A reference to a socket structure. This allows for multiple types.
	  */
  // Socket& _sock;
	 /** RTCP Parameter.
	  * Reference to a RTPSession object that is used to store RTCP state.
	  */
  RTPSession& _rtp_session;
	 /** Communication parameter.
	  * Maximum transmission unit.
	  */
  //  int _mtu;
 public:
	 /** Constructor.
	  * This calls the base constructor to create an RTPSocket.
	  */
  RTPSocket();
	 /** Constructor.
	  * This constructor creates a socket and sets the internal MTU member
	  * to be equal to the supplied parameter.
	  * @param s a reference to socket to be used for data transmission.
	  * @param rtp_session a reference to an RTPSession that will be used to
	  * store the protocol state.
	  * @param mtu RTP mtu size.
	  * store the protocol state.
	  */
  RTPSocket(Socket& s,
	    RTPSession& rtp_session,
	    const int& mtu);
	 /** Copy constructor.
	  * Copies values of other VRClientSocket into this object.
	  * @param s VRClientSocket to copy.
	  */
  RTPSocket(const RTPSocket &r);
	 /** Destructor. */	  
  virtual ~RTPSocket();
	 /** Transport function.
	  * Builds a DATA packet header and stores it in the supplied SocketBuffer
	  * @param sb array to store data header in.
	  * @param pt Payload type.
	  * @param sn Sequence Number.
	  * @param ts Timestamp.
	  * @param ssrc Syncrhonization Source.
	  * @param sz Size of the header.
	  * @param padding indicates whether or not padding is required
	  * @return 0: on success, -1 on error.
	  */
  int build_data_hdr(u_int8_t* sb, const u_int32_t& pt, 
		     const u_int16_t& sn, const u_int32_t& ts,
		     const u_int32_t& ssrc, const int& sz, 
		     bool padding);
	 /** Socket API method.
	  * Attempt to receive data into supplied buffer.
	  * @param b socket buffer to store data in.
	  * @param n number of bytes to attempt to receive.
	  * @return 0: on success, positive integer on success. On success
	  *         the value returned is the number of bytes read.
	  */
  int recv(u_int8_t* b, const int& n);
	 /** Socket API method.
	  * Attempt to send data in supplied buffer.NOTE: it is assumed that
	  * space for the header is pre-allocated in the buffer passed into
	  * this function at the beginning of the buffer. Otherwise, data
	  * will be overwritten. This is done for efficiency.
	  * @param b socket buffer to store data in.
	  * @param n number of bytes to attempt to receive.
	  * @return 0: on success, positive integer on success. On success
	  *         the value returned is the number of bytes sent.
	  */
  int send(u_int8_t* b, const int& n);
	 /** Socket API method.
	  * Binds socket to supplied address.
	  * @param ip IP address to bind to.
	  * @param port unused. 
	  * @return 0: on success, -1 on failure.
	  */
  int bind(const string& ip, const int& port=0);
	 /** Socket API method.
	  * Prepare a socket for accepting incoming connections.
	  * @return 0: on success, -1 on failure.
	  */
  int listen();
	 /** Socket API method.
	  * Wait for incoming connections on a socket. 
	  * @return -1: on failure, positive integer on success. On succes
	  *         the return value is a valid socket descriptor.
	  */
  int accept();
	 /** Socket API method.
	  * Attempt to establish a connection to the supplied address.
	  * @param ip IP address to connect to.
	  * @param port port to connect to.
	  * @return 0: on success, -1 on failure.
	  */
  int connect(const string& ip, const int& port);
	 /** FileSocket connection termination.
	  * Closes the file.
	  * @return 0: on success, -1 on failure.
	  */
  int close();
	 /** FileSocket connection termination.
	  * Closes the file.
	  * @return 0: on success, -1 on failure.
	  */
  int shutdown(const int& howto);
};

ostream& operator<<(ostream& os, const RTPSocket& s);

inline
int RTPSocket::build_data_hdr(u_int8_t* sb, const u_int32_t& pt, 
			      const u_int16_t& sn, const u_int32_t& ts,
			      const u_int32_t& ssrc, const int& sz, 
			      bool padding) {
         if (sz<HDR_SIZE)  return(-1);
	 RTPHdr* h=(RTPHdr*)sb;
	 h->_v=0x2;
	 h->_pt=pt;
	 h->_sn=htons(sn);
	 h->_ts=htonl(ts);
	 h->_ssrc=htonl(ssrc);
	 h->_p=padding?1:0;
	 return(0);
}

inline 
int RTPSocket::recv(u_int8_t* b, const int& n) {
  int ret=_sock.recv(b, n);
	 if (ret<0) {
		  syslog(LOG_ERR, "RTPSocket::recv(): error: %s", strerror(errno));
		  return(ret);
	 }
	 if (ret!=n) {
		  syslog(LOG_ERR, "RTPSocket::recv(): short read, ret==%d", ret);
		  return(ret);
	 }
	 // Process the packet.
	 RTPHdr* hdr=(RTPHdr*)b;
	 u_int16_t sn=ntohs(hdr->_sn);
	 u_int32_t ts=ntohl(hdr->_ts);
	 // Lock rtp_session.
	 // Mutex will be released when guard goes out of scope.
	 Guard rtp_session_guard(_rtp_session._mutex);
	 _rtp_session._ssrc=ntohl(hdr->_ssrc);
	 struct timeval last_sr, now, diff;
	 double arrival;
	 u_int32_t transit, d;
	 const double rtp_ts_interval=double(_rtp_session._packet_size)/
		  (double(_rtp_session._bits_per_sample)
			*double(_rtp_session._sampling_frequency*1e3));
	 
	 switch (hdr->_pt) {
	 case PT_DATA:
	 {
		  if (gettimeofday(&now, NULL)!=0) 
				syslog(LOG_ERR, "gettimeofday: %s", strerror(errno));
		  last_sr.tv_sec=_rtp_session._r._ntp_sec;
		  last_sr.tv_usec=_rtp_session._r._ntp_frac;
		  timersub(&now, &last_sr, &diff);
		  // Calculating packet jitter(from RFC3550).
		  // If Si is the RTP timestamp from packet i, and Ri is 
		  // the time of arrival in RTP timestamp units for packet 
		  // i, then for two packets i and j, D may be expressed 
		  // as :
		  // D(i,j) = (Rj - Ri) - (Sj - Si) = (Rj - Sj) - (Ri - Si)
		  // The interarrival jitter SHOULD be calculated 
		  // continuously as each data packet i is received from 
		  // source SSRC_n, using this difference D for that packet 
		  // and the previous packet i-1 in order of arrival (not 
		  // necessarily in sequence), according to the formula
		  // J(i) = J(i-1) + (|D(i-1,i)| - J(i-1))/16
		  // Get time since last SR.
		  double ddiff=diff.tv_sec+diff.tv_usec/1e6;
		  // Convert to units of rtp_ts.
		  ddiff/=rtp_ts_interval;
		  // Add offset.
		  ddiff+=_rtp_session._r._rtp_ts;
		  arrival=ddiff;
		  transit = (u_int32_t)(arrival - ts);
		  d = transit - _rtp_session._r._transit;
		  if (d<0) d = -d;
		  _rtp_session._r._jitter += (1./16.)*((double)d - 
															_rtp_session._r._jitter);
		  syslog(LOG_DEBUG, "RTPSocket::recv(): sn max_seq==%d %d", 
					sn, _rtp_session._r._max_seq);
		  _bytes_rcvd+=ret;
		  // FIXME No retransmissions.
		  // TODO: incorporate configurable lossy + lossless 
		  // updating into here.
		  if (_rtp_session.update_seq(sn)==1) {
				syslog(LOG_DEBUG, "Accept packet: sn==%d", sn);
		  } else {
				syslog(LOG_DEBUG, "Discard packet: sn==%d", sn);
		  }
	 }
	 break;
	 default:
		  syslog(LOG_ERR, "RTPData::recv(): Received unknown packet");
		  return(-1);
		  break;
	 }
	 return(ret);
}

inline
int RTPSocket::send(u_int8_t* b, const int& n) 
{
	 // We have something to send.
	 // IMPORTANT. For efficiency's sake, we assume space has been reserved
	 // for the header in the buffer passed to us. Important to make sure that
	 // this is the case!

	 // Lock rtp_session.
	 // Mutex will be released when guard goes out of scope.
	 Guard rtp_session_guard(_rtp_session._mutex);
	 const u_int8_t pt=PT_DATA;
	 if (build_data_hdr(b, 
							  pt, 
							  _rtp_session._s._sn_max, 
							  _rtp_session._s._rtp_ts,
							  _rtp_session._ssrc, HDR_SIZE, 
							  false)!=0) {
		  syslog(LOG_ERR, "RTPSocket::send(): unable to build data header");
		  return(-1);
	 }
	 int ret=_sock.send(b, n);
	 if (ret<0) {
		  syslog(LOG_ERR, "RTPSocket::send(): send error: %s",
					strerror(errno));
		  return(ret);
	 }
	 if (ret!=n) {
		  syslog(LOG_ERR, "RTPSocket::send(): short send. ret==%d", ret);
	 }
	 // Valid send.
	 _bytes_sent+=ret;
	 ++_rtp_session._s._rtp_ts;
	 ++_rtp_session._s._psent;
	 _rtp_session._s._osent+=(ret-HDR_SIZE);
	 syslog(LOG_DEBUG, "RTPSocket::send(): sn_min sn_max==%d %d\n",         
			  _rtp_session._s._sn_min,
			  _rtp_session._s._sn_max);
	 // Update the _sn_max state.
	 ++_rtp_session._s._sn_max;
	 return(ret);
}

#endif

