/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2004 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifdef LINUX
#include <unistd.h>
#endif
// #include <fcntl.h>
// #include <syslog.h>
// #include <netinet/in.h>
#include <arpa/inet.h>

#include <Socket.h>

const int Socket::BLOCKING=0x1;
const int Socket::NONBLOCKING=0x10;
const int Socket::SEL_READ=0x1;
const int Socket::SEL_WRITE=0x10;
const int Socket::SEL_EXCEPT=0x100;

Socket::Socket():
  _sockd(-1), _mtu(1024), 
  _bytes_sent(0),_bytes_rcvd(0),
  _send_calls(0), _recv_calls(0),_timeout(10)
{
  _age.start();
}

Socket::Socket(const int mtu):
  _sockd(-1), _mtu(mtu), 
  _bytes_sent(0),_bytes_rcvd(0),
  _send_calls(0), _recv_calls(0),_timeout(10)
{
  _age.start();
}

Socket::Socket(const int& s, const int m):
  _sockd(s), _mtu(m), 
  _bytes_sent(0),_bytes_rcvd(0),
  _send_calls(0), _recv_calls(0),_timeout(10)
{
  _age.start();
}

Socket::Socket(const Socket& s):
  _sockd(s._sockd), _mtu(s._mtu),
  _bytes_sent(s._bytes_sent), _bytes_rcvd(s._bytes_rcvd),
  _send_calls(s._send_calls), _recv_calls(s._recv_calls),_timeout(10)
{
}

Socket::~Socket() 
{
	 _age.stop();
}

string Socket::get_peer_ip(const int& sd)
{
	 struct sockaddr_in name;
	 socklen_t namelen;
	 if (getpeername(sd, (struct sockaddr*)&name, &namelen)!=0) {
		  syslog(LOG_ERR, "Socket::get_peer_ip: %s", strerror(errno));
	 }
	 string ip(inet_ntoa(name.sin_addr));
	 return(ip);
}

void Socket::set_mtu(const int m){ _mtu =m;}

// Member access.

int Socket::get_mtu() const {
	 return(_mtu);
}

long long Socket::get_bytes_sent() const {
	 return(_bytes_sent);
}   

long long Socket::get_bytes_rcvd() const {
	 return(_bytes_rcvd);
}

int Socket::get_send_calls() const {
	 return(_send_calls);
}

int Socket::get_recv_calls() const {
	 return(_recv_calls);
}

// Options.
int Socket::set_so_reuseaddr(const int yes)
{
  return
    ::setsockopt(_sockd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int));
}

int Socket::get_so_reuseaddr() const
{
	 int so_reuseaddr;
	 socklen_t optlen=sizeof(int);
	 int ret=getsockopt(_sockd, SOL_SOCKET, SO_REUSEADDR, &so_reuseaddr,
							  &optlen);
	 if (ret==-1)
		  return(-1);
	 return(so_reuseaddr);
}

int Socket::set_so_linger(const int& onoff, const int& time)
{
	 struct linger so_linger;
	 so_linger.l_onoff=onoff;
	 so_linger.l_linger=time;
    return(::setsockopt(_sockd, SOL_SOCKET, SO_LINGER, &so_linger, 
								sizeof(struct linger)));
}

const struct linger Socket::get_so_linger() const
{        
  struct linger so_linger;
  socklen_t optlen=sizeof(struct linger);
  int ret=::getsockopt(_sockd, SOL_SOCKET, SO_LINGER, &so_linger,
		       &optlen);
  if (ret==-1) {
    syslog(LOG_ERR, "Socket::get_so_linger(): unable to getsockopt(): %s",
	   strerror(errno));
    so_linger.l_onoff=-1;
    so_linger.l_linger=-1;
  }
  return(so_linger);
}

int Socket::set_so_rcvbuf(const int& sz)
{
  return(::setsockopt(_sockd, SOL_SOCKET, SO_RCVBUF, &sz,
		      sizeof(int)));	
}

int Socket::get_so_rcvbuf() const
{
  int so_rcvbuf;
  socklen_t optlen=sizeof(int);
  int ret=::getsockopt(_sockd, SOL_SOCKET, SO_RCVBUF, &so_rcvbuf,
		       &optlen);
  if (ret==-1)
    return(-1);
  return(so_rcvbuf);
}

int Socket::set_so_sndbuf(const int  & sz)
{
  return
    ::setsockopt(_sockd, SOL_SOCKET, SO_SNDBUF, &sz, sizeof(int));	
}

int  Socket::get_so_sndbuf() const
{
  int so_sndbuf;
  socklen_t optlen=sizeof(int);
  int ret=::getsockopt(_sockd, SOL_SOCKET, SO_SNDBUF, &so_sndbuf,
		       &optlen);
  if (ret==-1)
    return(-1);
  return(so_sndbuf);
}

int Socket::set_so_rcvlowat(const int& m)
{
  return(::setsockopt(_sockd, SOL_SOCKET, SO_RCVLOWAT, &m,
		      sizeof(int)));	
}

int Socket::get_so_rcvlowat() const
{
  int so_rcvlowat;
  socklen_t optlen=sizeof(int);
  int ret=::getsockopt(_sockd, SOL_SOCKET, SO_RCVLOWAT, &so_rcvlowat,
		       &optlen);
  if (ret==-1)
    return(-1);
  return(so_rcvlowat);
}

int Socket::set_so_sndlowat(const int& m)
{
  return(::setsockopt(_sockd, SOL_SOCKET, SO_SNDLOWAT, &m,
		      sizeof(int)));
}

int Socket::get_so_sndlowat() const
{
  int so_sndlowat;
  socklen_t optlen=sizeof(int);
  int ret=::getsockopt(_sockd, SOL_SOCKET, SO_SNDLOWAT, &so_sndlowat,
		       &optlen);
  if (ret==-1)
    return(-1);
  return(so_sndlowat);
}

int Socket::set_tcp_nodelay(const int& n)
{
  return(::setsockopt(_sockd, IPPROTO_TCP, TCP_NODELAY, &n,
		      sizeof(int)));
}

int Socket::get_tcp_nodelay() const
{
  socklen_t optlen=sizeof(int);
  int tcp_nodelay;
  int ret=::getsockopt(_sockd, IPPROTO_TCP, TCP_NODELAY, &tcp_nodelay,
		       &optlen);
  if (ret==-1)
    return(-1);
  return(tcp_nodelay);
}

// Blocking.
int Socket::set_blocking(const int& b)
{
	 int ret=0;
	 int blocking_flags=fcntl(_sockd, F_GETFL, 0);
	 if (b==NONBLOCKING) {
		  blocking_flags |= O_NONBLOCK;
		  ret=fcntl(_sockd, F_SETFL, blocking_flags);
	 } 
	 if (b==BLOCKING) {
		  blocking_flags &= ~O_NONBLOCK;
		  ret=fcntl(_sockd, F_SETFL, blocking_flags);
	 }
	 if (ret!=-1)
		  ret=0;
	 return(ret);	
}

int Socket::get_blocking() const
{
	 int blocking_flags=fcntl(_sockd, F_GETFL, 0);
	 if (blocking_flags==-1)
		  return(-1);
	 if (blocking_flags & O_NONBLOCK)
		  return(NONBLOCKING);
    return(BLOCKING);
}

int Socket::set_ip_tos(const int& t) 
{
	 return(::setsockopt(_sockd, IPPROTO_IP, IP_TOS, &t, sizeof(int)));
}

int Socket::get_ip_tos() const
{
	 int ip_tos;
	 socklen_t optlen=sizeof(int);
	 int ret=::getsockopt(_sockd, IPPROTO_IP, IP_TOS, &ip_tos, &optlen);
	 if (ret==-1)
		  return(-1);
	 return(ip_tos);
}


