/* 
 * This software is released under the terms of the MIT License(included below). 
 *  
 * Copyright (c) 2003 MIT Haystack Observatory 
 *  
 * Permission is hereby granted, free of charge, to any person obtaining a   
 * copy of this software and associated documentation files (the "Software"),    
 * to deal in the Software without restriction, including without limitation   
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,   
 * and/or sell copies of the Software, and to permit persons to whom the  
 * Software is furnished to do so, subject to the following conditions: 
 *  
 * The above copyright notice and this permission notice shall be included in    
 * all copies or substantial portions of the Software. 
 *   
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE  
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER   
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 * 
 */

#ifndef TEST_SOCKET_H
#define TEST_SOCKET_H

#include <common.h>
#include <Socket.h>
#include <list>
#include <string>
#include <iostream>
#include <syslog.h>

/** TestSocket class
 * @doc This class encapsulates a Test stream.
 */

/** TestSocket class
 * @doc This class is used for testing and TVG. It inherits from the Socket
 * class and provides methods that closely match those of the BSD sockets
 * API. The main aim of this class is to act as a data source/sink.
 */
class TestSocket: public Socket 
{
  protected:
	 /** Buffer.
	  * Buffer for storing fill pattern.
	  */
	 u_int8_t* _buf;
  public:
	 /** Default constructor.
	  * This initializes the data structure, but does *not* create a socket.
	  */
    TestSocket();
	 /** Constructor.
	  * This constructor creates a socket and sets the internal MTU member
	  * to be equal to the supplied parameter.
	  * @param m MTU for this socket.
	  */
    TestSocket(const int &m);
	 /** Copy constructor.
	  * Copies values of other TestSocket into this object.
	  * @param s TestSocket to copy.
	  */
    TestSocket(const TestSocket &s);
	 /** Constructor.
	  * This constructor is provided two initizialization parameters.
	  * @param s socket descriptor to encapsulate.
	  * @param m MTU for this socket.
	  */
    TestSocket(const int& s, const int& m);
	 /** Destructor. */
    virtual ~TestSocket();
	 /** Assigment operator.
	  * Copies supplied socket fields into this object.
	  * @param s socket to copy.
	  */
    TestSocket& operator=(const TestSocket& s);
	 /** Access method.
	  * Allows setting of TestSocket's internal socket descriptor. This is
	  * useful for allowing a TestSocket object to encapsulate a socket
	  * descriptor that has been returned from another function, e.g.
	  * accept().
	  * @param s socket descriptor to encapsulate.
	  */
    void set_sockd(const int& s);
	 /** Access method.
	  * Allows seting of TestSocket's internal mtu size.
	  * @param m mtu size.
	  */
	 void set_mtu(const int& m);
	 /** Socket API method.
	  * Binds socket to supplied address.
	  * @param ip IP address to bind to.
	  * @param port port to bind to.
	  * @return 0: on success, -1 on failure.
	  */
    int bind(const string& ip, const int& port);
	 /** Socket API method.
	  * Prepare a socket for accepting incoming connections.
	  * @return 0: on success, -1 on failure.
	  */
    int listen();
	 /** Socket API method.
	  * Wait for incoming connections on a socket. 
	  * @return -1: on failure, positive integer on success. On succes
	  *         the return value is a valid socket descriptor.
	  */
    int accept();
	 /** Socket API method.
	  * Attempt to establish a connection to the supplied address.
	  * @param ip IP address to connect to.
	  * @param port port to connect to.
	  * @return 0: on success, -1 on failure.
	  */
    int connect(const string& ip, const int& port);
	 /** Socket API method.
	  * Attempt to receive data into supplied buffer.
	  * @param b socket buffer to store data in.
	  * @param n number of bytes to attempt to receive.
	  * @return 0: on success, positive integer on success. On success
	  *         the value returned is the number of bytes read.
	  */
    int recv(u_int8_t* b, const int& n);
	 /** Socket API method.
	  * Attempt to send data in supplied buffer.
	  * @param b socket buffer to store data in.
	  * @param n number of bytes to attempt to receive.
	  * @return 0: on success, positive integer on success. On success
	  *         the value returned is the number of bytes sent.
	  */
    int send(u_int8_t* b, const int& n);
    /** Socket API method.
	  * @doc shutdown connection as per standard socket library.
	  * @param how method to use to shutdown socket: SHUT_RD, SHUT_WR, 
	  * SHUT_RDWR.
	  */
    int shutdown(const int& howto);
	 /** Socket API method.
	  * @doc close socket connection.
	  */
    int close();
};

ostream& operator<<(ostream& os, const TestSocket& s);

inline 
int TestSocket::recv(u_int8_t* b, const int& n)
{
	 int sz=_mtu>n?n:_mtu;
	 memcpy(b, _buf, sz);
	 ++_recv_calls;
	 _bytes_rcvd+=sz;
	 return(sz);
}

inline 
int TestSocket::send(u_int8_t* b, const int& n)
{
	 ++_send_calls;
	 _bytes_sent+=n;
	 return(n);
}

#endif

